///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Author: JT Cook - http://www.avalondreams.com
//
// Last Modified: 6.24.09
//
// Description: XGS-Pac - VGA version
//
// Todo:
// change speed of pac and ghosts as levels progress, have pac round the corners quicker/wider
//
// New:
// Created VGA version
// Move score to top and level and number of guys to the bottom
// fixed bug where you could keep eating the score from the fruit and racking up points
// -----
// added bonus score when you eat a ghost or fruit
// fixed pink ghost graphic facing down
// fixed bug when ghosts/player go in tunnel their sprite would be partially shown on the far right hand of the screen
// fixed bug where ghosts would get stuck in the tunnel
// -----
// fixed ghost logic where ghosts get confused sometimes
// bonus fruit 
// Ghosts slow down in tunnel
// Pac slows down when he eats pellets
// Death animation
// Score and hi-score added
// in game text (game over, ready, score)
// power pellets (so you can eat ghosts)
// round counter displayed
// blink power pellets
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//#define PC_SIM 
#define XGS_CONSOLE 
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#ifdef PC_SIM
#include "xgs_wrapper2.h" //for PC Dev
#endif

#ifdef XGS_CONSOLE
// Include main system header
#include "XGS_PIC_SYSTEM_V010.h"

// Include graphics drivers
#include "JTC_GFX_DRV_v001.h"
#include "JTC_VGA_TILE_SPRITE_V001.h"

// Include the gamepad driver
#include "XGS_PIC_GAMEPAD_DRV_V010.h"
#endif 

#include "pac_gfx_03.h" //graphics
#include "font1bit.h" //1 bit font
#include "ntsc_to_vga.h"
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define font_offset 1 //where the font tiles starts
#define font_start 48 //where in the ascii chart the font starts
//sprite offset values
#define SpriteX_off -2
#define SpriteY_off -2+16
// Generates a random integer between x and y inclusive
#define RAND_RANGE(x,y) ( ( x ) + (rand()%( ( y ) - ( x )+1)))
#define Maze_Color 0x4C
//#define White_Maze 0xF1
#define White_Maze 0xBF
#define New_Maze_Color 0x4C
#define font_color 0x3F
#define font_bg 0x00
#define free_life_score 10000 //score you have to get to get a free life
#define start_lives 3
#define bonus_time_fruit 60*10
//#define chunky 1 //use normal tiles or chunky tiles (to minimize color bleeding)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS
#define dot_delay 3;
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Sprite g_Sprites[MAX_SPRITES];					// Max sprites
MapScroll g_Map[1];  //Map scrolling info
// The TILE map buffer must be allocated into far memory space because 
// it is so large it wont fit into near 
unsigned char g_TileMap[SCREEN_TILE_MAP_WIDTH*SCREEN_TILE_MAP_HEIGHT] __attribute__((far)) = {};
unsigned char g_SpriteData[256*8] __attribute__((far)); //sprites from g_PacSprites
unsigned char g_TileData[(64)*46] __attribute__((far)); //tiles
//Maze map - 21x24
unsigned char const pac_maze[SCREEN_TILE_MAP_WIDTH*SCREEN_TILE_MAP_HEIGHT] __attribute__((far)) = {
//19 is tunnel tile that slows down the ghosts
//20 invisable wall
//13 is the door to the ghost hut
///*
10, 7, 7, 7, 7, 7, 7, 7, 7, 7,16, 7, 7, 7, 7, 7, 7, 7, 7, 7, 9,
 8, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8,
 8, 2,10, 7, 9, 1,10, 7, 9, 1, 8, 1,10, 7, 9, 1,10, 7, 9, 2, 8,
 8, 1,11, 7,12, 1,11, 7,12, 1, 6, 1,11, 7,12, 1,11, 7,12, 1, 8,
 8, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 8,
 8, 1, 4, 7, 3, 1, 5, 1, 4, 7,16, 7, 3, 1, 5, 1, 4, 7, 3, 1, 8,
 8, 1, 1, 1, 1, 1, 8, 1, 1, 1, 8, 1, 1, 1, 8, 1, 1, 1, 1, 1, 8,
11, 7, 7, 7, 9, 1,15, 7, 3, 1, 6, 1, 4, 7,14, 1,10, 7, 7, 7,12,
 0, 0, 0, 0, 8, 1, 8, 0, 0, 0, 0, 0, 0, 0, 8, 1, 8, 0, 0, 0, 0,
 0, 0, 0, 0, 8, 1, 8, 0,10, 7,13, 7, 9, 0, 8, 1, 8, 0, 0, 0, 0,
 7, 7, 7, 7,12, 1, 6, 0, 8,20,19,20, 8, 0, 6, 1,11, 7, 7, 7, 7,
19,19,19,19,19, 1, 0, 0, 8,20,19,20, 8, 0, 0, 1,19,19,19,19,19,
 7, 7, 7, 7, 9, 1, 5, 0, 8,20,19,20, 8, 0, 5, 1,10, 7, 7, 7, 7,
 0, 0, 0, 0, 8, 1, 8, 0,11, 7, 7, 7,12, 0, 8, 1, 8, 0, 0, 0, 0,
 0, 0, 0, 0, 8, 1, 8, 0, 0, 0, 0, 0, 0, 0, 8, 1, 8, 0, 0, 0, 0,
10, 7, 7, 7,12, 1, 6, 0, 4, 7,16, 7, 3, 0, 6, 1,11, 7, 7, 7, 9,
 8, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8,
 8, 1, 4, 7, 9, 1, 4, 7, 3, 1, 6, 1, 4, 7, 3, 1,10, 7, 3, 1, 8,
 8, 2, 1, 1, 8, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 8, 1, 1, 2, 8,
15, 7, 3, 1, 6, 1, 5, 1, 4, 7,16, 7, 3, 1, 5, 1, 6, 1, 4, 7,14,
 8, 1, 1, 1, 1, 1, 8, 1, 1, 1, 8, 1, 1, 1, 8, 1, 1, 1, 1, 1, 8,
 8, 1, 4, 7, 7, 7,17, 7, 3, 1, 6, 1, 4, 7,17, 7, 7, 7, 3, 1, 8,
 8, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8,
11, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,12,
//*/
/* //debug maze
10, 7, 7, 7, 7, 7, 7, 7, 7, 7,16, 7, 7, 7, 7, 7, 7, 7, 7, 7, 9,
 8, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8,
 8, 2,10, 7, 9, 1,10, 7, 9, 1, 8, 1,10, 7, 9, 1,10, 7, 9, 2, 8,
 8, 1,11, 7,12, 1,11, 7,12, 9, 6, 9,11, 7,12, 1,11, 7,12, 1, 8,
 8, 1, 1, 1, 1, 1, 1, 1, 0, 9, 0, 9, 1, 1, 0, 1, 1, 1, 1, 1, 8,
 8, 1, 4, 7, 3, 1, 5, 1, 4, 7, 0, 7, 3, 1, 5, 1, 4, 7, 3, 1, 8,
 8, 1, 1, 1, 1, 1, 8, 1, 1, 9, 0, 9, 1, 1, 8, 1, 1, 1, 1, 1, 8,
11, 7, 7, 7, 9, 1,15, 7, 3, 7, 0, 7, 4, 7,14, 1,10, 7, 7, 7,12,
 0, 0, 0, 0, 8, 1, 8, 0, 0, 9, 0, 9, 0, 0, 8, 1, 8, 0, 0, 0, 0,
 0, 0, 0, 0, 8, 1, 8, 7,10, 7, 9, 7, 9, 7, 8, 1, 8, 0, 0, 0, 0,
 7, 7, 7, 7,12, 1, 6, 0, 8,10,19,10, 8, 0, 6, 1,11, 7, 7, 7, 7,
19,19,19,19,19, 1, 0, 0, 8,10,19,10, 8, 0, 0, 1,19,19,19,19,19,
 7, 7, 7, 7, 9, 1, 5, 0, 8,10,19,10, 8, 0, 5, 1,10, 7, 7, 7, 7,
 0, 0, 0, 0, 8, 1, 8, 0,11, 7, 7, 7,12, 0, 8, 1, 8, 0, 0, 0, 0,
 0, 0, 0, 0, 8, 1, 8, 0, 0, 0, 0, 0, 0, 0, 8, 1, 8, 0, 0, 0, 0,
10, 7, 7, 7,12, 1, 6, 0, 4, 7,16, 7, 3, 0, 6, 1,11, 7, 7, 7, 9,
 8, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8,
 8, 1, 4, 7, 9, 1, 4, 7, 3, 1, 6, 1, 4, 7, 3, 1,10, 7, 3, 1, 8,
 8, 2, 1, 1, 8, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 8, 1, 1, 2, 8,
15, 7, 3, 1, 6, 1, 5, 1, 4, 7,16, 7, 3, 1, 5, 1, 6, 1, 4, 7,14,
 8, 1, 1, 1, 1, 1, 8, 1, 1, 1, 8, 1, 1, 1, 8, 1, 1, 1, 1, 1, 8,
 8, 1, 4, 7, 7, 7,17, 7, 3, 1, 6, 1, 4, 7,17, 7, 7, 7, 3, 1, 8,
 8, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8,
11, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,12,
*/
};

//pacman animation
unsigned const char pac_animation[]={0,1,2,1,0,3,4,3,0,5,6,5,0,7,8,7};
//bonus score table
unsigned const int bonus_score[]={100,300,500,500,700,700,1000,1000,2000,2000,3000,3000,5000};
//
unsigned const int score_gfx[] = {0,1,2,2,3,3,4,4,5,5,6,6,7};
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void print_string(int x, int y, char *string);
void In_Game();
int check_wall(unsigned char x,unsigned char y,unsigned char xx,unsigned char yy);
int move_pac(int p_dir);
void move_ghost(unsigned char g_number);
void reset_characters(); 
void reset_maze();
void check_dots(int x, int y);
void Beat_Level();
void change_maze(unsigned char old_color, unsigned char new_color);
void copy_font(int char_number, int char_data);
void Menu();
void intro_screen();
void copy_sprites(unsigned char *source_sprite,unsigned char *dest_sprite);
void draw_lives();
int pac_ghost_hit();
void get_game_chars();
void draw_score();
void draw_hi_score();
int check_path(unsigned char x, unsigned char y,unsigned char xx,unsigned char yy, unsigned char ghost_number);
void pac_fruit_hit();
unsigned char GamepadValue, GamepadValueOld;
unsigned char GameState; //state of game (ie: menu, game play, scrolling, etc)
unsigned char pac_frame=0, pac_tic=0, free_life;
unsigned char pac_x, pac_y, pac_x_old, pac_y_old, pac_lives;
unsigned int pac_dir, pac_dir_old, ghost_timer;
unsigned int pac_score, hi_score;
unsigned char ghost_x[4], ghost_y[4], ghost_x_old[4], ghost_y_old[4], ghost_dir[4], ghost_tic[4], ghost_frame[4], ghost_hall_ctr[4];
unsigned char ghost_edible[4], ghost_toggle, level, ghost_eaten, ghost_delay,pac_eat_delay, bonus_flag;
unsigned int total_dots, misc_timer, pellet_timer, pellet_status, low_bonus, hi_bonus, bonus_timer, seed_counter=0;
void draw_char(int message);
unsigned const char ghost_blue_time[] = {4,3,2,1,0,3,0,0,0,3,0,0,0,1,0,0,0,0,0};
unsigned const char ghost_blink_number[] = {5,5,5,5,5,5,5,5,3,5,5,3,3,3,5,3,1,3,1};
unsigned int score_timer[5];
// Our main start location
#ifdef XGS_CONSOLE
int main (void)
#endif
#ifdef PC_SIM
void main_init ()
#endif
{

	int i;
	// Always call ConfigureClock first
	SYS_ConfigureClock(FCY_VGA);

	// Init the gamepad
	Gamepad_Init();

	// Init the graphics system
    GFX_InitTile(g_TileData, g_TileMap, g_Sprites, g_Map);

    fill_screen(0); //clears the screen
    hi_score=0;

    //move all sprites off screen
    for(i=0;i<8;i++)
    {
		g_Sprites[i].X = 0;
		g_Sprites[i].Y = 0;
    }


	// Start up our display interrupt
	GFX_StartDrawing(SCREEN_TYPE_VGA);
	g_Map[0].Y=(256-16);
	GameState=0; 
	intro_screen();
#ifdef XGS_CONSOLE
        main_loop();
	return(0);
#endif
}


void main_loop()
{
#ifdef XGS_CONSOLE
  while(1)
#endif 
  {
	// Wait until we are in v-sync to do any processing
	WAIT_FOR_VSYNC_START();

	switch(GameState)
	{
		//intro screen
		case 0:
			Menu();
		break;
		//main game
		case 1:
			In_Game();
		break;
		//beat the level
		case 2:
			Beat_Level();
		break;
		//pause/ready right before a level
		case 3:
		   draw_char(1);//draw ready message
  		   misc_timer++;
		   if(misc_timer>120)
  		   {
		     GameState=1; 
   		     misc_timer=0;
		     draw_char(0);//clear ready message
		   }
		break;
		//pac dies
		case 4:
		   misc_timer++;
	           //death animation
		   if(pac_frame<12)
		   {		
		     if(misc_timer>6)
		     {
			if(pac_frame<11)
			 copy_sprites((g_PacSprites + ((46+pac_frame)<<8)), g_SpriteData);
			else
			 g_Sprites[0].X=0; //move pac off screen
			pac_frame++;
			misc_timer=0;
			
		     }
                   }
		   //pause while dead
		   else 
		   {
			if(misc_timer>120)
			{
			 pac_frame=0;
			 if(pac_lives>0)
			 {	
			  pac_lives--;
			  GameState=3;
			  misc_timer=0;
			  reset_characters();		
			 }
			 else
			 {
			  misc_timer=0;
			  GameState=5; //Game Over	
			 }
			}
                   }
		break;		
		//game over
		case 5:
		   draw_char(2); //game over message
		   misc_timer++;
		   if(misc_timer>120)
		   {
			  intro_screen();
			  misc_timer=0;
			  GameState=0; //Game Over	
                   }
		break;		

	};
	// Wait here till we are done with v-sync
	WAIT_FOR_VSYNC_END();

  }
}

void Menu()
{
		int i, i2;
		seed_counter++;
		GamepadValue = Gamepad_Read(0);		
		// player controls
                //right, left, up, down
		if(GamepadValue & GAMEPAD_ACTION1 | GamepadValue & GAMEPAD_ACTION2 | GamepadValue & GAMEPAD_START)
		{
			//seed randomizer
			srand(seed_counter);
                       	GameState=3;
				i2=0;
				//use chunky tiles?
				#ifdef chunky
				 i2=19<<6; 
				#endif 
			    //copy maze tiles
			    for(i=0;i<(64*19)-1;i++)
			    {
				   g_TileData[i]=ntsc_to_vga_lut[pac_TileData[i+i2]];
			    }
			    pac_score=0; //reset score
			    free_life=0 ;//reset free life counter    
			    level=0; //reset level
			    reset_maze(); //draws the maze
    			    reset_characters(); //place all the characters/sprites	
			    get_game_chars(); //grab select characters for in game text
		}
		pac_x--;
		//move pac at the bottom of the screen
		g_Sprites[0].Y=pac_y+SpriteY_off;
 		g_Sprites[0].X=pac_x+SpriteX_off;
		g_Sprites[0].X&=0xFF;
		g_Sprites[0].Y&=0xFF;
		pac_tic++;
		//animate pac
		if(pac_tic>4)
		{
			pac_frame++;
			pac_frame&=3;
			//g_Sprites[0].PixelData = g_SpriteData + (9 <<8); //debug
			pac_tic=0;			
		}		
		i=(pac_animation[pac_frame+(pac_dir<<2)]);
		if(i>9)
			i=9;
                
		copy_sprites((g_PacSprites + (i<<8)), g_SpriteData);
		g_Sprites[0].PixelData = g_SpriteData;
}

//main gameplay
void In_Game()
{
    int i=0, j;
	{
		//delay to slow down ghosts if it hits the tunnel
    		ghost_delay^=1; 
		//blink pellet
		pellet_timer++;
		if(pellet_timer>29)
		{
			pellet_status^=1;
			pellet_timer=0;
		    #ifdef chunky
			  j=(pellet_status<<7) + (19<<6); 
			#else
			  j=pellet_status<<7;
		    #endif 
			for(i=0;i<64;i++)
			{
				g_TileData[i+128]=ntsc_to_vga_lut[pac_TileData[i+j]];
			}
			
		}
		pac_dir_old=pac_dir;		
		// Read the gamepad
		GamepadValue = Gamepad_Read(0);		
		// player controls
                //right, left, up, down
		if(GamepadValue & GAMEPAD_DOWN)
		{
                       	pac_dir=3;
		}
		else if(GamepadValue & GAMEPAD_UP)
		{
				pac_dir=2;
		}
		else if(GamepadValue & GAMEPAD_LEFT)
		{
				pac_dir=1;
		}
		else if(GamepadValue & GAMEPAD_RIGHT)
		{
			    pac_dir=0;
		}

		//move pac and sees if it hits a wall
		j=move_pac(pac_dir);
		if(j>0 && (pac_dir_old != pac_dir))
		{
		    pac_dir=pac_dir_old;
  		    j=move_pac(pac_dir);
		}
		else if(j==0) //doesn't hit any walls
		{
		    if(pac_eat_delay<1)
		    {	
	 	     if(pac_dir!=pac_dir_old)
			pac_tic=99;
		     //make sure we stay aligned

		     if(pac_dir==2 || pac_dir==3) //up or down
		     {
///*
			 if((pac_x&0x07)>3)
		            pac_x+=8;
			 pac_x&=0xF8;
//*/
/*
                         if(pac_dir_old==1)
                         {
			  if((pac_x&0x07)<4)
		          //    pac_x+=8;
		              pac_x-=8;
                         }
                         else if(pac_dir_old==0)
                         {
			  if((pac_x&0x07)>3)
		              pac_x+=8;
                         }
                      pac_x&=0xF8;
*/
		     }
		     if(pac_dir==0 || pac_dir==1) //left or right
		     {
///*
			 if((pac_y&0x07)>3)
		            pac_y+=8;
			 pac_y&=0xF8;
//*/
/*
                         if(pac_dir_old==2)
                         {
			  if((pac_y&0x07)<4)
		          //    pac_x+=8;
		              pac_y-=8;
                         }
                         else if(pac_dir_old==3)
                         {
			  if((pac_y&0x07)>3)
		              pac_y+=8;
                         }
                      pac_y&=0xF8;
*/
		     }

		    }
		    else
		    {
			pac_x=pac_x_old;
			pac_y=pac_y_old;			
			pac_eat_delay--;
		    }
		}

		g_Sprites[0].Y=pac_y+SpriteY_off;
 		g_Sprites[0].X=pac_x;//+SpriteX_off; //game crashes here if we add a value to pac_x, memory allignment issue?
 		g_Sprites[0].X+=SpriteX_off;
		g_Sprites[0].X&=0xFF;
		g_Sprites[0].Y&=0xFF;
		//animate pac
		if(pac_tic>4)
		{
			pac_frame++;
			pac_frame&=3;
			//g_Sprites[0].PixelData = g_SpriteData + (9 <<8); //debug
			pac_tic=0;			
		}		
		j=(pac_animation[pac_frame+(pac_dir<<2)]);
		if(j>9)
			j=9;
                
		copy_sprites((g_PacSprites + (j<<8)), g_SpriteData);
		g_Sprites[0].PixelData = g_SpriteData;

                if(ghost_edible[0]<2)
                 move_ghost(0); 
                if(ghost_edible[1]<2)
                 move_ghost(1); 
                if(ghost_edible[2]<2)
                 move_ghost(2); 
                if(ghost_edible[3]<2)
                 move_ghost(3); 

		//handle bonus fruit
		if(bonus_timer>0)
		{
		  g_Sprites[5].Y=129+SpriteY_off;
 		  g_Sprites[5].X=96+SpriteX_off;
		  bonus_timer--;
		  if(bonus_timer==0 && score_timer[4]==0)
		  { 
                    //Move bonus fruit off screen
                    g_Sprites[5].Y=0;
		  }
		}
		
		//handle bonus score when something is eaten
		for(j=0;j<5;j++)
                {
		  if(score_timer[j]>0)
		  {
		    score_timer[j]--;
		    if(score_timer[j]==0)
                    {
                     //bonus fruit
                     if(j==4)
                     {
                      //Move bonus fruit off screen
                      g_Sprites[5].Y=0; 
                      //put fruit sprite back as graphic
		      copy_sprites(g_PacSprites + (57<<8), g_SpriteData +(5<<8));		
                     }
		     //ghosts
                     else
		     {
                        ghost_edible[j]=0;
			ghost_x[j]=96;
			//allign ghost in the maze
			ghost_y[j]=100;
			if((ghost_x[j]&0x07)>3)
		             ghost_x[j]+=8;
			 ghost_x[j]&=0xF8;
			if((ghost_y[j]&0x07)>3)
		            ghost_y[j]+=8;
			ghost_y[j]&=0xF8;
			ghost_dir[j]=2; 
		     }
		    }
                  }
                }
                //handle ghost timer
		if(ghost_timer>0)
                {
		  ghost_timer--;
		  if(ghost_timer==0)
		  {
		   //flash the ghost
                   ghost_toggle++;
		   j=level;
		   if(j>18)
		      j=18;
                
                   if(ghost_toggle<ghost_blink_number[j]+1)
                   {
			ghost_timer=15;
                   }
		   //we are done with ghost being edible
		   else
		   {
		     ghost_eaten=0;
		     ghost_toggle=0;
                     for(j=0;j<4;j++)
		       if(ghost_edible[j]<2)
		        ghost_edible[j]=0;
		   }
		  }
                }
		pac_fruit_hit(); //check fruit colision detection
		j=pac_ghost_hit();
  		//j=0; //debug - pac can't get hit
		//ghost kills pac
		if(j>0 && ghost_edible[j-1]==0)
		{
			pac_frame=0;
			misc_timer=0;
			GameState=4;
		}
		//pac eats ghost
		else if(j>0 && ghost_edible[j-1]==1)
		{
			ghost_edible[j-1]=2;
			//add score multiplier 
			ghost_eaten++;
     			//draw bonus score for ghost
			score_timer[j-1]=60*2;

			if(ghost_eaten==1)
                        {
			  pac_score+=200;
			  copy_sprites(g_PacSprites + ((58+8)<<8), g_SpriteData +((j)<<8));		
                        }
			else if(ghost_eaten==2)
                        {
			  pac_score+=400;
			  copy_sprites(g_PacSprites + ((58+9)<<8), g_SpriteData +((j)<<8));		
                        }
			else if(ghost_eaten==3)
                        {
			  pac_score+=800;
			  copy_sprites(g_PacSprites + ((58+10)<<8), g_SpriteData +((j)<<8));		
                        }
			else if(ghost_eaten==4)
                        {
			  pac_score+=1600;
			  copy_sprites(g_PacSprites + ((58+11)<<8), g_SpriteData +((j)<<8));		
                        }
			draw_score();
		}

	}

}

void intro_screen()
{
    copy_font(0,0); //copy font tiles
    fill_screen(0); //clears the screen
    print_string(10, 1,"XGS-PAC");
    print_string( 9,15,"Hit Start");
	print_string(0,21,"   www.xgamestation.com");
    print_string(0,23,"   www.avalondreams.com");
    print_string(0,24,"         JT Cook");
    //print_string(0,23,"xxxxxxxxxxxxxxxxxxxxxxxxxx");    

    ghost_dir[0]=0;
    ghost_frame[0]=0;
    copy_sprites(g_PacSprites + ((21-7)<<8) +((ghost_dir[0]<<1) + ghost_frame[0]<<8) + ((0<<3)<<8), g_SpriteData + (1<<8));
    g_Sprites[1].PixelData = g_SpriteData + (1<<8);
    g_Sprites[1].X=36+SpriteX_off;
    g_Sprites[1].Y=44+SpriteY_off;

    copy_sprites(g_PacSprites + ((21-7)<<8) +((ghost_dir[0]<<1) + ghost_frame[0]<<8) + ((1<<3)<<8), g_SpriteData + (2<<8));
    g_Sprites[2].PixelData = g_SpriteData + (3<<8);
    g_Sprites[2].X=36+SpriteX_off;
    g_Sprites[2].Y=60+SpriteY_off;

    copy_sprites(g_PacSprites + ((21-7)<<8) +((ghost_dir[0]<<1) + ghost_frame[0]<<8) + ((2<<3)<<8), g_SpriteData + (3<<8));
    g_Sprites[3].PixelData = g_SpriteData + (2<<8);
    g_Sprites[3].X=36+SpriteX_off;
    g_Sprites[3].Y=76+SpriteY_off;

    copy_sprites(g_PacSprites + ((21-7)<<8) +((ghost_dir[0]<<1) + ghost_frame[0]<<8) + ((3<<3)<<8), g_SpriteData + (4<<8));
    g_Sprites[4].PixelData = g_SpriteData + (4<<8);
    g_Sprites[4].X=36+SpriteX_off;
    g_Sprites[4].Y=92+SpriteY_off;
/*
    print_string(7,4,  "Shadow .... Blinky");
    print_string(7,6,  "Speedy .... Pinky");
    print_string(7,8,  "Bashful ... Inky");
    print_string(7,10, "Pokey ..... Clyde");
*/
    print_string(5,4,  "Chance .... Stinky");
    print_string(5,6,  "Haphazard.. Dinky");
    print_string(5,8,  "Randy...... Infringy");
    print_string(5,10, "Unplanned.. Mortimer");

    ghost_frame[0]=1;
    copy_sprites(g_PacSprites +((ghost_dir[0]<<1) + ghost_frame[0]<<8), g_SpriteData + (0<<8));
    g_Sprites[0].PixelData = g_SpriteData + (0<<8);
    g_Sprites[0].X=36+SpriteX_off;
    g_Sprites[0].Y=162+SpriteY_off;

    pac_dir=1;
    pac_x=26+8;
    pac_y=160;

    pac_lives=start_lives; // starting lives
}

void draw_hi_score()
{
 char score_text[10];
 int n, screen_ctr, x,y;
 sprintf(score_text,"%7.i",hi_score);
 //x=22;
 //y=3; 
 x=0;
 y=31;
 //offset=font_offset-font_start;
 screen_ctr=y*SCREEN_TILE_MAP_WIDTH+x;
 for(n=0;n<8;n++)
 {
   //g_TileMap[screen_ctr]=n+19+15;
   //offset is 34
   //make sure we are only printing characters that we have graphics for
   if(score_text[n]>=48 && score_text[n]<58)
    g_TileMap[screen_ctr]=score_text[n]-12; //draw digit
   else if(score_text[n]==0)
    return;
   screen_ctr++;
 }

}

//draw score in game
void draw_score()
{
 char score_text[10];
 int n, screen_ctr, x,y;
 sprintf(score_text,"%7.i",pac_score);
 //x=22;
 //y=7;
 x=12;
 y=31;
 screen_ctr=y*SCREEN_TILE_MAP_WIDTH+x;
 //check hi score
 if(pac_score>hi_score)
 {
   hi_score=pac_score;
   draw_hi_score();
 }
 
 //handle free life
 if(free_life==0 && pac_score>free_life_score)
 {
  free_life=1; //set free life counter so they can only get 1 per game
  pac_lives++;
  draw_lives();
 }
 for(n=0;n<8;n++)
 {
   //g_TileMap[screen_ctr]=n+19+15;

   //offset is 34
   //make sure we are only printing characters that we have graphics for
   if(score_text[n]>=48 && score_text[n]<58)
    g_TileMap[screen_ctr]=score_text[n]-12; //draw digit
   else if(score_text[n]==0)
    return;

   screen_ctr++;
 }

}

//draw in game text messages
void draw_char(int message)
{
 #define game_font_start 19
 int n, offset;
// READY!GMOVRHISCUN
// 01234567890123456
//           1111111

 char game_over_text[]={6,2,7,1,99,8,9,1,10};
 char score_text[]={13,14,8,10,1};
 char hi_score_text[]={11,12,13,14,8,10,1};
 char round_text[]={0,8,15,16,3};
 //draw no message
 if(message==0)
 {
  offset=14*SCREEN_TILE_MAP_WIDTH+8;
  for(n=0;n<6;n++)
      g_TileMap[offset+n]=0;
 }
 //draw ready message
 else if(message==1)
 {
  offset=14*SCREEN_TILE_MAP_WIDTH+8;
  for(n=0;n<6;n++)
      g_TileMap[offset+n]=n+game_font_start;
 }
 //draw game over message
 else if(message==2)
 {
  offset=14*SCREEN_TILE_MAP_WIDTH+6;
  for(n=0;n<9;n++)
     if(game_over_text[n]<99)
      g_TileMap[offset+n]=game_over_text[n]+game_font_start;
 }
 //draw hiscore
 else if(message==3)
 {
 // offset=2*SCREEN_TILE_MAP_WIDTH+22;
 offset=30*SCREEN_TILE_MAP_WIDTH;
  for(n=0;n<7;n++)
     //if(game_over_text[n]<99)
      g_TileMap[offset+n]=hi_score_text[n]+game_font_start;
 }
 //draw score
 else if(message==4)
 {
 // offset=6*SCREEN_TILE_MAP_WIDTH+22;
 offset=30*SCREEN_TILE_MAP_WIDTH+12;
  for(n=0;n<5;n++)
      g_TileMap[offset+n]=score_text[n]+game_font_start;
 }
 
 //draw round
 else if(message==5)
 {
  //draw round text
//  offset=19*SCREEN_TILE_MAP_WIDTH+22;
  offset=24*SCREEN_TILE_MAP_WIDTH+(22-10);
  for(n=0;n<5;n++)
      g_TileMap[offset+n]=round_text[n]+game_font_start;

  //draw round number
  sprintf(score_text,"%i",(level+1));
  //offset=20*SCREEN_TILE_MAP_WIDTH+22;
  offset=24*SCREEN_TILE_MAP_WIDTH+(22-4);
  for(n=0;n<3;n++)
  {
    if(score_text[n]>=48 && score_text[n]<58)
     g_TileMap[offset+n]=score_text[n]-12; //draw digit
    else if(score_text[n]==0)
     return;

  }

 }
}

//get characters used in game (like game over, ready, etc)
void get_game_chars()
{
 int n, nx, ny, n2, offset;
 char t_char;
 char char_set[]={82,69,65,68,89,33,71,77,79,86,82,72,73,83,67,85,78};
// READY!GMOVRHISCUN
// 01234567890123456
//           1111111

  for(n=0;n<17;n++)
  {
   offset=(8*char_set[n]);
   n2=((game_font_start+n)<<6);
   for(ny=0;ny<8;ny++)
   {
    t_char=font1bit[offset];
    offset++;
    for(nx=0;nx<8;nx++)
    {
     if(t_char & 0x80)
        g_TileData[n2]=font_color;
     else
        g_TileData[n2]=font_bg;
     t_char<<=1;
     n2++;
    }
   }
  }
  //copy numbers
  for(n=0;n<10;n++)
  {
   offset=(8*(48+n));
   n2=((game_font_start+n+17)<<6);
   //n2=((game_font_start+n)<<6);
   for(ny=0;ny<8;ny++)
   {
    t_char=font1bit[offset];
    offset++;
    for(nx=0;nx<8;nx++)
    {
     if(t_char & 0x80)
        g_TileData[n2]=font_color;
     else
        g_TileData[n2]=font_bg;
     t_char<<=1;
     n2++;
    }
   }
  }

}

void reset_maze()
{
//copy maze
    int i, mapx=0, map_offset=0;
    fill_screen(0); //clears the screen
    change_maze(Maze_Color,New_Maze_Color);
    total_dots=0; //clear pellet counter
    for(i=0;i<21*(24);i++)
    {
	   //slow part of the maze
	   if(pac_maze[i]<19)
	      g_TileMap[map_offset+mapx]=pac_maze[i];
           //invisible wall
	   else if(pac_maze[i]==20)
	      g_TileMap[map_offset+mapx]=0;
           //tunnel to slow down the ghosts
           else
	      g_TileMap[map_offset+mapx]=0;
	   //count up total dots that need to be eaten
	   if(pac_maze[i]==1 || pac_maze[i]==2)
	      total_dots++;
	   mapx++;
	   if(mapx>20)
           {
		mapx=0;
		map_offset+=SCREEN_TILE_MAP_WIDTH;
	   }
    }
    low_bonus=(total_dots*7)/10; //30%
    //low_bonus=total_dots-5; //debug
    hi_bonus=(total_dots*3)/10;  //70%
    bonus_flag=0;
    draw_char(3); //draw hi score
    draw_char(4); //score
    draw_char(5); //round number
    draw_hi_score(); //draw hi score number
    draw_score(); //draw score number
}

//draw lives remaining
void draw_lives()
{
 int i;
 for(i=0;i<6;i++)
 {
  if(i+1<=pac_lives)
//   g_TileMap[((15)*SCREEN_TILE_MAP_WIDTH)+22+i]=18;
   g_TileMap[((24)*SCREEN_TILE_MAP_WIDTH)+i]=18;
  else
//   g_TileMap[((15)*SCREEN_TILE_MAP_WIDTH)+22+i]=0;
   g_TileMap[((24)*SCREEN_TILE_MAP_WIDTH)+i]=0;
 }
}

void reset_characters()
{
//reset Pacman, all the ghosts (after death, new game, new level)
		int j;
		pac_x =95;
		pac_y =128+32;
                pac_x_old=pac_x;
		pac_y_old=pac_y;
		g_Sprites[0].Y=pac_y+SpriteY_off;
		g_Sprites[0].X=pac_x+SpriteX_off;
		pac_dir=1;
		pac_frame=0;
		pac_dir_old=pac_dir;
		j=(pac_animation[pac_frame+(pac_dir<<2)]);
		if(j>9)
			j=9;
                copy_sprites(g_PacSprites + (j<<8), g_SpriteData);
		g_Sprites[0].PixelData = g_SpriteData;

                //setup ghosts
		ghost_x[0]=95;
		//ghost_y[0]=80;
		ghost_y[0]=102;
		ghost_dir[0]=2;
		g_Sprites[1].X=ghost_x[0]+SpriteX_off;
		g_Sprites[1].Y=ghost_y[0]+SpriteY_off;
		ghost_edible[0]=0;
 		//-----
		ghost_x[1]=95;
		ghost_y[1]=80;
		ghost_dir[1]=RAND_RANGE(0,3);
		copy_sprites(g_PacSprites + ((21-7)<<8) +((ghost_dir[0]<<1) + ghost_frame[0]<<8) + ((0<<3)<<8), g_SpriteData + (1<<8));
		g_Sprites[1].PixelData = g_SpriteData + (1<<8);
		g_Sprites[2].X=ghost_x[1]+SpriteX_off;
		g_Sprites[2].Y=ghost_y[1]+SpriteY_off;
		ghost_edible[1]=0;		
		//-----
		ghost_x[2]=95-16;
		ghost_y[2]=80;
		ghost_dir[2]=RAND_RANGE(0,3);
                copy_sprites(g_PacSprites + ((21-7)<<8) +((ghost_dir[1]<<1) + ghost_frame[1]<<8) + ((1<<3)<<8), g_SpriteData + (2<<8));
		g_Sprites[2].PixelData = g_SpriteData + (2<<8);
		g_Sprites[3].X=ghost_x[2]+SpriteX_off;
		g_Sprites[3].Y=ghost_y[2]+SpriteY_off;
		ghost_edible[2]=0;
		//------
		ghost_x[3]=95+16;
		ghost_y[3]=80;
		ghost_dir[3]=RAND_RANGE(0,3);
		copy_sprites(g_PacSprites + ((21-7)<<8) +((ghost_dir[2]<<1) + ghost_frame[2]<<8) + ((2<<3)<<8), g_SpriteData + (3<<8));
		g_Sprites[3].PixelData = g_SpriteData + (3<<8);
		g_Sprites[4].X=ghost_x[3]+SpriteX_off;
		g_Sprites[4].Y=ghost_y[3]+SpriteY_off;		
		ghost_edible[3]=0;
		copy_sprites(g_PacSprites + ((21-7)<<8) +((ghost_dir[3]<<1) + ghost_frame[3]<<8) + ((3<<3)<<8), g_SpriteData + (4<<8));
		g_Sprites[4].PixelData = g_SpriteData + (4<<8);
		
		//setup bonus fruit
		copy_sprites(g_PacSprites + (57<<8), g_SpriteData +(5<<8));		
		g_Sprites[5].Y=0;
		g_Sprites[5].PixelData = g_SpriteData + (5<<8);

		//bonus score timers
		for(j=0;j<5;j++)
                  score_timer[j]=0;

		draw_lives();
		ghost_toggle=0;
		ghost_eaten=0;		
		pellet_timer=0;
		pellet_status=0;
		pac_eat_delay=0;
		bonus_timer=0;
}

unsigned char check_ghost_direction(unsigned char g_number)
{
   unsigned char wall_check1=0, wall_check2, wall_check=0;
   switch(ghost_dir[g_number])
   {
	case 0: // right (check up and down)
          //check up
          wall_check1=check_wall(ghost_x[g_number]+2,ghost_y[g_number]+2-4,ghost_x[g_number]+7-2,ghost_y[g_number]+7-2-4);
	  //check down
          wall_check2=check_wall(ghost_x[g_number]+2,ghost_y[g_number]+2+4,ghost_x[g_number]+7-2,ghost_y[g_number]+7-2+4);
	break;
	case 1: // left (check up and down)
          //check up
          wall_check1=check_wall(ghost_x[g_number]+2,ghost_y[g_number]+2-4,ghost_x[g_number]+7-2,ghost_y[g_number]+7-2-4);
	  //check down
          wall_check2=check_wall(ghost_x[g_number]+2,ghost_y[g_number]+2+4,ghost_x[g_number]+7-2,ghost_y[g_number]+7-2+4);
	break;
	case 2: // up (check left and right)
          //check left
          wall_check1=check_wall(ghost_x[g_number]+2-4,ghost_y[g_number]+2,ghost_x[g_number]+7-2-4,ghost_y[g_number]+7-2);
	  //check right
          wall_check2=check_wall(ghost_x[g_number]+2+4,ghost_y[g_number]+2,ghost_x[g_number]+7-2+4,ghost_y[g_number]+7-2);
	break;
	case 3: // left (check left and right)
          //check left
          wall_check1=check_wall(ghost_x[g_number]+2-4,ghost_y[g_number]+2,ghost_x[g_number]+7-2-4,ghost_y[g_number]+7-2);
	  //check right
          wall_check2=check_wall(ghost_x[g_number]+2+4,ghost_y[g_number]+2,ghost_x[g_number]+7-2+4,ghost_y[g_number]+7-2);
	break;
   }   
   if(wall_check1<2)
      wall_check|=1; //mark wall check 1
   if (wall_check2<2)
      wall_check|=2; //mark wall check 2

 return(wall_check);
}

unsigned char check_random_dir(unsigned char dir1, unsigned char dir2, unsigned char dir3)
{
 //bit 1 = right, bit 2 = left, bit 3 = up, bit 4 = down
 unsigned char dir_value;
 //only 2 directions
 if(dir3>3)
 {
   dir_value=RAND_RANGE(0,1);
   if(dir_value==0)
     return(dir1);
   else
     return(dir2);
 }
 else
 {
   dir_value=RAND_RANGE(0,2);
   if(dir_value==0)
     return(dir1);
   else if(dir_value==1)
     return(dir2);
   else
     return(dir3);
 }
}

void move_ghost(unsigned char g_number)
{
 char wall_check, ghost_dir_old;
//ghost_x[4], ghost_y[4], ghost_x_old[4], ghost_y_old[4], ghost_dir[4], ghost_tic[4], ghost_frame[4];
 ghost_x_old[g_number]=ghost_x[g_number];
 ghost_y_old[g_number]=ghost_y[g_number];
 ghost_dir_old=ghost_dir[g_number];
 switch(ghost_dir[g_number])
 {
    case 0: //right
     ghost_x[g_number]++;
    break;
    case 1: //left
     ghost_x[g_number]--;
    break;
    case 2: //up
     ghost_y[g_number]--;
    break;
    case 3: //down
     ghost_y[g_number]++;
    break;
 }

  //check screen limits for tunnels
  if(ghost_x[g_number]<2) //+16-3
     ghost_x[g_number]=((20<<3)+14);
  else if(ghost_x[g_number]>((20<<3)+13))
     ghost_x[g_number]-=((20<<3)+12);

 //if ghost hits a wall, change direction
 if(check_path(ghost_x[g_number]+1,ghost_y[g_number]+1,ghost_x[g_number]+7-1,ghost_y[g_number]+7-1,g_number)>1)
 {
   ghost_x[g_number]=ghost_x_old[g_number];
   ghost_y[g_number]=ghost_y_old[g_number];
   wall_check=check_ghost_direction(g_number);
   switch(ghost_dir[g_number])
   {
//x1 = RAND_RANGE(0,(SCREEN_WIDTH/2)-1);		// Random X in half the screen
	case 0: //right
	  if((wall_check&3) == 3) //both up and down are free (randomly select)
	    ghost_dir[g_number]=RAND_RANGE(2,3);
	  else if((wall_check&1) == 1) //up is free
	    ghost_dir[g_number]=2;
	  else if((wall_check&2) == 2) //down is free
	    ghost_dir[g_number]=3;
	  else //backtrack
	    ghost_dir[g_number]=1; 
	break;
	case 1: //left
	  if((wall_check&3) == 3) //both up and down are free (randomly select)
	    ghost_dir[g_number]=RAND_RANGE(2,3);
	  else if((wall_check&1) == 1) //up is free
	    ghost_dir[g_number]=2;
	  else if((wall_check&2) == 2) //down is free
	    ghost_dir[g_number]=3;
	  else //backtrack
	    ghost_dir[g_number]=0; 
	break;
	case 2: //up
	  if((wall_check&3) == 3) //both up and down are free (randomly select)
	    ghost_dir[g_number]=RAND_RANGE(0,1);
	  else if((wall_check&1) == 1) //left is free
	    ghost_dir[g_number]=1;
	  else if((wall_check&2) == 2) //right is free
	    ghost_dir[g_number]=0;
	  else //backtrack
	    ghost_dir[g_number]=3; 
	break;
	case 3: //down
	  if((wall_check&3) == 3) //both up and down are free (randomly select)
	    ghost_dir[g_number]=RAND_RANGE(0,1);
	  else if((wall_check&1) == 1) //left is free
	    ghost_dir[g_number]=1;
	  else if((wall_check&2) == 2) //right is free
	    ghost_dir[g_number]=0;
	  else //backtrack
	    ghost_dir[g_number]=2; 
	break;
   }
   if(ghost_dir[g_number]==2 || ghost_dir[g_number]==3) //up or down
   {
	if((ghost_x[g_number]&0x07)>3)
		ghost_x[g_number]+=8;
	ghost_x[g_number]&=0xF8;
   }
   if(ghost_dir[g_number]==0 || ghost_dir[g_number]==1) //left or right
   {
	if((ghost_y[g_number]&0x07)>3)
		ghost_y[g_number]+=8;
	ghost_y[g_number]&=0xF8;
   }
 }
 //check to see if there are any open passages around the ghosts
 else if(ghost_hall_ctr[g_number]>11)
 {
   wall_check=check_ghost_direction(g_number);
   switch(ghost_dir[g_number])
   {
	case 0: //right
	  if((wall_check&3) == 3) //both up and down are free (randomly select)
 	    ghost_dir[g_number]=check_random_dir(0, 2, 3);
	  else if((wall_check&1) == 1) //up is free
 	    ghost_dir[g_number]=check_random_dir(0, 2, 9);
	  else if((wall_check&2) == 2) //down is free
 	    ghost_dir[g_number]=check_random_dir(0, 3, 9);
	break;
	case 1: //left
	  if((wall_check&3) == 3) //both up and down are free (randomly select)
 	    ghost_dir[g_number]=check_random_dir(1, 2, 3);
	  else if((wall_check&1) == 1) //up is free
 	    ghost_dir[g_number]=check_random_dir(1, 2, 9);
	  else if((wall_check&2) == 2) //down is free
 	    ghost_dir[g_number]=check_random_dir(1, 3, 9);
	break;
	case 2: //up
	  if((wall_check&3) == 3) //both up and down are free (randomly select)
 	    ghost_dir[g_number]=check_random_dir(2, 0, 1);
	  else if((wall_check&1) == 1) //left is free
 	    ghost_dir[g_number]=check_random_dir(2, 1, 9);
	  else if((wall_check&2) == 2) //right is free
 	    ghost_dir[g_number]=check_random_dir(2, 0, 9);
	break;
	case 3: //down
	  if((wall_check&3) == 3) //both up and down are free (randomly select)
 	    ghost_dir[g_number]=check_random_dir(3, 0, 1);
	  else if((wall_check&1) == 1) //left is free
 	    ghost_dir[g_number]=check_random_dir(3, 1, 9);
	  else if((wall_check&2) == 2) //right is free
 	    ghost_dir[g_number]=check_random_dir(3, 0, 9);
	break;

   }

   if(ghost_dir[g_number]==2 || ghost_dir[g_number]==3) //up or down
   {
	if((ghost_x[g_number]&0x07)>3)
		ghost_x[g_number]+=8;
	ghost_x[g_number]&=0xF8;
   }
   if(ghost_dir[g_number]==0 || ghost_dir[g_number]==1) //left or right
   {
	if((ghost_y[g_number]&0x07)>3)
		ghost_y[g_number]+=8;
	ghost_y[g_number]&=0xF8;
   }
   ghost_hall_ctr[g_number]=0;
  // if the ghost is in the tunnel, slow it down 
  if(check_path(ghost_x[g_number]+1,ghost_y[g_number]+1,ghost_x[g_number]+7-1,ghost_y[g_number]+7-1,g_number)>0 && ghost_delay==1)
  {
   ghost_x[g_number]=ghost_x_old[g_number];
   ghost_y[g_number]=ghost_y_old[g_number];
  }

 }
 else
 {
  ghost_hall_ctr[g_number]++;
  // if the ghost is in the tunnel, slow it down
  if(check_path(ghost_x[g_number]+1,ghost_y[g_number]+1,ghost_x[g_number]+7-1,ghost_y[g_number]+7-1,g_number)>0 && ghost_delay==1)
  {
   ghost_x[g_number]=ghost_x_old[g_number];
   ghost_y[g_number]=ghost_y_old[g_number];
  }
 }
 g_Sprites[1+g_number].X=ghost_x[g_number]+SpriteX_off;
 g_Sprites[1+g_number].Y=ghost_y[g_number]+SpriteY_off;

 ghost_tic[g_number]++;
 if(ghost_tic[g_number]>8)
 {
   ghost_tic[g_number]=0;
   ghost_frame[g_number]^=1;
   //update graphic if frame changes
   //normal ghost
   if(ghost_edible[g_number]==0)
     copy_sprites(g_PacSprites + ((21-7)<<8) +((ghost_dir[g_number]<<1) + ghost_frame[g_number]<<8) + ((g_number<<3)<<8), g_SpriteData + ((g_number+1)<<8) );
   //ghost can be eaten
   else
     copy_sprites(g_PacSprites + (10<<8)+((ghost_toggle&1)<<9) + (ghost_frame[g_number]<<8), g_SpriteData + ((g_number+1)<<8) );
 }

//update graphic if ghost changes direction
   //normal ghost
   if(ghost_edible[g_number]==0)
     copy_sprites(g_PacSprites + ((21-7)<<8) +((ghost_dir[g_number]<<1) + ghost_frame[g_number]<<8) + ((g_number<<3)<<8), g_SpriteData + ((g_number+1)<<8) );
   //ghost can be eaten
   else
     copy_sprites(g_PacSprites + (10<<8)+((ghost_toggle&1)<<9) + (ghost_frame[g_number]<<8), g_SpriteData + ((g_number+1)<<8) );


}

//////////////////////////////////////////////
//check fruit colision detection
void pac_fruit_hit()
{
  unsigned char bonus_cnt;
/*
  if((pac_x+2<=g_Sprites[5].X+10) && (pac_x+10>=g_Sprites[5].X+2) &&
     (pac_y+2<=g_Sprites[5].Y+10) && (pac_y+10>=g_Sprites[5].Y+2))
*/
  if((g_Sprites[0].X+2<=g_Sprites[5].X+10) && (g_Sprites[0].X+10>=g_Sprites[5].X+2) &&
     (g_Sprites[0].Y+2<=g_Sprites[5].Y+10) && (g_Sprites[0].Y+10>=g_Sprites[5].Y+2) &&
	  bonus_timer>0)
  {
     bonus_timer=0;
     //g_Sprites[5].Y=0; //move bonus off screen
     bonus_cnt=level;
     if(bonus_cnt>12)
        bonus_cnt=12;
     pac_score+=bonus_score[bonus_cnt];
     draw_score();
     
     //draw bonus score for fruit
     score_timer[4]=60*2;
     //bonus score sprite
     copy_sprites(g_PacSprites + ((58+score_gfx[bonus_cnt])<<8), g_SpriteData +(5<<8));		
  }
}
 
//////////////////////////////////////////////
//check hit detection between pac and ghost
int pac_ghost_hit() 
{
 int gnumber;

 for(gnumber=0;gnumber<4;gnumber++)
 {
  if((pac_x+2<=ghost_x[gnumber]+10) && (pac_x+10>=ghost_x[gnumber]+2) &&
     (pac_y+2<=ghost_y[gnumber]+10) && (pac_y+10>=ghost_y[gnumber]+2))
	return(gnumber+1); //return ghost that was hit
 }

 return(0); //no hit
}

//////////////////////////////////////////////
// move pac on map
int move_pac(int p_dir)
{
		int j;
		pac_x_old=pac_x;
		pac_y_old=pac_y;

		//move pac
		if(p_dir==0) //right
		{
				pac_x++;
/*
				if(pac_dir_old==2)
				    j=check_wall(pac_x, pac_y+2, pac_x+7, pac_y+2);
				else if(pac_dir_old==3)
				    j=check_wall(pac_x, pac_y+6, pac_x+7, pac_y+6);
				else
*/
				    j=check_wall(pac_x, pac_y+4, pac_x+7, pac_y+4);
		}
		else if(p_dir==1) //left
		{
				pac_x--;
/*
				if(pac_dir_old==2)
				    j=check_wall(pac_x, pac_y+2, pac_x+7, pac_y+2);
				else if(pac_dir_old==3)
				    j=check_wall(pac_x, pac_y+6, pac_x+7, pac_y+6);
				else
*/
				    j=check_wall(pac_x, pac_y+4, pac_x+7, pac_y+4);

		}
		else if(p_dir==2) //up
		{
				pac_y--;
/*
				if(pac_dir_old==0)
				    j=check_wall(pac_x+6, pac_y, pac_x+6, pac_y+7);
				else if(pac_dir_old==1)
				    j=check_wall(pac_x+2, pac_y, pac_x+2, pac_y+7);
				else
*/
				    j=check_wall(pac_x+4, pac_y, pac_x+4, pac_y+7);

		}
		else if(p_dir==3) //down
		{
				pac_y++;
/*
				if(pac_dir_old==0)
				    j=check_wall(pac_x+6, pac_y, pac_x+6, pac_y+7);
				else if(pac_dir_old==1)
				    j=check_wall(pac_x+2, pac_y, pac_x+2, pac_y+7);
				else
*/
				    j=check_wall(pac_x+4, pac_y, pac_x+4, pac_y+7);
		}
		//check colision detection
		if(j>0)
		{
			  pac_x=pac_x_old;
			  pac_y=pac_y_old;
		}
		else
                {
		 //check screen limits for tunnels
		 if(pac_x<2) //+16-3
		   pac_x=((20<<3)+14);
		 else if(pac_x>((20<<3)+13))
		   pac_x-=((20<<3)+12);

			pac_tic++;
			//check dots
			check_dots(pac_x, pac_y);

                }

		return(j);		
}

//////////////////////////////////////////////
// Check colision detection
int check_wall(unsigned char x,unsigned char y,unsigned char xx,unsigned char yy)
{
 //#define x_limit_max 233
 #define x_limit_max (20<<3)
 #define x_limit_min 6
 int x1, x2, y1,y2, hit=0;
 
 x-=16;
 y-=16;
 if(x>x_limit_max)
//   x-=x_limit_max;
  x=0;
//  x=x_limit_max;
 if(y>191)
   y=191;

 xx-=16;
 yy-=16;
 if(xx>x_limit_max)
//   xx-=x_limit_max;
//  xx=x_limit_max;
  xx=0;

 if(yy>191)
   yy=191;

 x1=x;
 y1=y;
 x2=xx;
 y2=yy;
 x1>>=3;
 y1>>=3;
 x2>>=3;
 y2>>=3;
 hit=0;
 //upper left, bit 0
 if(g_TileMap[(y1*SCREEN_TILE_MAP_WIDTH)+x1]>2)
    hit|=1;
 //upper right, bit 1
 if(g_TileMap[(y1*SCREEN_TILE_MAP_WIDTH)+x2]>2)
    hit|=2;
 //lower left, bit 2
 if(g_TileMap[(y2*SCREEN_TILE_MAP_WIDTH)+x1]>2)
    hit|=4;
 //lower right, bit 3
 if(g_TileMap[(y2*SCREEN_TILE_MAP_WIDTH)+x2]>2)
    hit|=8;

 return(hit);
}

//////////////////////////////////////////////
// Check colision detection (on maze map)
int check_path(unsigned char x, unsigned char y,unsigned char xx,unsigned char yy, unsigned char ghost_number)
{
 #define x_limit_max 20<<3
 #define x_limit_min 6
 int x1, x2, y1,y2, hit=0, value1,value2,value3,value4;
 x-=16;
 y-=16;

 if(x>x_limit_max)
   //x-=x_limit_max;
   x=0;
 if(y>191)
   y=191;
 xx-=16;
 yy-=16;

 if(xx>x_limit_max)
   //xx-=x_limit_max;
   xx=0;
 if(yy>191)
   yy=191;
/*
 if(x<x_limit_min)
    x=x_limit_min;
 if(xx<x_limit_min)
    xx=x_limit_min;
*/
 x1=x;
 y1=y;
 x2=xx;
 y2=yy;
 x1>>=3;
 y1>>=3;
 x2>>=3;
 y2>>=3;
 hit=0;
//Maze map - 21x24
//unsigned char const pac_maze[SCREEN_TILE_MAP_WIDTH*SCREEN_TILE_MAP_HEIGHT] __attribute__((far)) = {
 //upper left, bit 0
 value1=pac_maze[(y1*21)+x1];
 value2=pac_maze[(y1*21)+x2];
 value3=pac_maze[(y1*21)+x1];
 value4=pac_maze[(y2*21)+x2];
 //check ghost wall
 if(value1==20 || value2==20 || value3==20 || value4==20)
 {
    hit=2;
    //exit(0);
 }
 //check value to slow down ghost (tunnel)
 else if(value1==19 || value2==19 || value3==19 || value4 ==19)
    hit=1;
 //door at ghost maze
 else if(value1==13 || value2==13 || value3 ==13 || value4==13 )
 {
    hit=1;
    ghost_dir[ghost_number]=2; //make sure ghost always points up
 }
 //wall
 else if(value1>2 || value2>2 || value3>2 || value4>2)
 {
    hit=2;
 }
 //everything else is awall 
 return(hit);
}

void get_power_pellet()
{
 int n=0, timer;
 //timer before ghosts stop blinking
 timer=level;
 if(timer>18)
  timer=18;
 ghost_toggle=0;
 ghost_timer=(ghost_blue_time[timer]*60)+2;
 //set all the ghosts so you can eat them
 for(n=0;n<4;n++)
   ghost_edible[n]=1;
}

void check_dots(int x, int y)
{
 int tempx, tempy;
 x-=16;
 y-=16;
 tempx=x&7;
 tempy=y&7;
 if((tempx<3 || tempx>5) && (tempy<3 || tempy>5))
 {
  if(tempx>5)
     x+=8;
  if(tempy>5)
     y+=8;
  // check dot
  x>>=3;
  y>>=3;
  tempx=(y*SCREEN_TILE_MAP_WIDTH)+x;
  //check limits
//  if(tempx>(SCREEN_TILE_MAP_WIDTH * SCREEN_TILE_MAP_HEIGHT)-1)
//     tempx=(SCREEN_TILE_MAP_WIDTH * SCREEN_TILE_MAP_HEIGHT)-1;
  //check dot
  if(g_TileMap[tempx]==1)
  {
     //eat dot
     g_TileMap[tempx]=0;
     total_dots--;
     pac_score+=10;
     draw_score();
     //slow down pac since he ate a dot
     pac_eat_delay=dot_delay;
     //check for bonus fruit
     if(total_dots==low_bonus && bonus_flag==0)
     {
      bonus_flag=1;
      bonus_timer=bonus_time_fruit;
     }
     else if(total_dots==hi_bonus && bonus_flag==1)
     {
      bonus_flag=2;
      bonus_timer=bonus_time_fruit;
     }
  }
  //check power pellet
  else if(g_TileMap[tempx]==2)
  {
     //eat power pellet
     get_power_pellet();
     g_TileMap[tempx]=0;
     total_dots--;
     pac_score+=50;
     draw_score();
     //slow down pac since he ate a dot
     pac_eat_delay=dot_delay;
  }
  //all gone
  if(total_dots==0)
  {
    GameState=2;
    misc_timer=0; 
    level++;
  }
 }
}
///////////////////////////////////////////////////////////////
//What to do when you beat the maze and eat all the dots
void Beat_Level()
{
 //use the dot counter as a time
 total_dots++;

 if(total_dots>30)
 {
   misc_timer++;
   total_dots=0;
   pac_dir^=1;
   pac_dir&=1;
   if(pac_dir==1)
        //change the maze to white
        change_maze(New_Maze_Color, White_Maze);
   else
        //change the maze to blue
        change_maze(White_Maze, New_Maze_Color);

  if(misc_timer>6)
  {
   //make sure maze ends up blue
   change_maze(White_Maze, New_Maze_Color);
   GameState=3; 
   reset_maze();
   reset_characters();
   misc_timer=0;
  }
 }
}

//////////////////////////////////////////////////////////////
//screen helpers
void change_maze(unsigned char old_color, unsigned char new_color)
{
 unsigned char tiles, mul;
 unsigned int n=64*3;
 for(tiles=0;tiles<15;tiles++)
 {
  for(mul=0;mul<64;mul++)
  {
    if(g_TileData[n]==ntsc_to_vga_lut[old_color])
       g_TileData[n]=ntsc_to_vga_lut[new_color];
    n++;
  }
 }
}

//////////////////////////////////////////////////////////////
//screen helpers
void copy_sprites(unsigned char *source_sprite, unsigned char *dest_sprite)
{
 unsigned char sprite_color;
 unsigned int cnt, n=0;
 for(cnt=0;cnt<256;cnt++)
 {
   dest_sprite[n]=ntsc_to_vga_lut[source_sprite[n]];
   n++;
 }
}

void copy_font(int char_number, int char_data)
{
 //font_color, font_bg
 int n=0,n2=0, nx, ny, offset=8*48;
 unsigned char t_char;
 for(n=0;n<64;n++)
 {
  g_TileData[n2]=font_bg;
  n2++;
 }
 
 //numbers
 for(n=0;n<10;n++)
 {
  for(ny=0;ny<8;ny++)
  {
   t_char=font1bit[offset];
   offset++;
   for(nx=0;nx<8;nx++)
   {
    if(t_char & 0x80)
       g_TileData[n2]=font_color;
    else
       g_TileData[n2]=font_bg;
    t_char<<=1;
    n2++;
   }
  }
 }
 //letters
 offset=8*65;
 for(n=0;n<26;n++)
 {
  for(ny=0;ny<8;ny++)
  {
   t_char=font1bit[offset];
   offset++;
   for(nx=0;nx<8;nx++)
   {
    if(t_char & 0x80)
       g_TileData[n2]=font_color;
    else
       g_TileData[n2]=font_bg;
    t_char<<=1;
    n2++;
   }
  }
 }
  //period
  offset=8*46;
  for(ny=0;ny<8;ny++)
  {
   t_char=font1bit[offset];
   offset++;
   for(nx=0;nx<8;nx++)
   {
    if(t_char & 0x80)
       g_TileData[n2]=font_color;
    else
       g_TileData[n2]=font_bg;
    t_char<<=1;
    n2++;
   }
  }

}

//prints a string
void print_string(int x, int y, char *string)
{
 int n, offset, screen_ctr;
 offset=font_offset-font_start;
 screen_ctr=y*SCREEN_TILE_MAP_WIDTH+x;
 for(n=0;n<50;n++)
 {
   if(string[n]>=48 && string[n]<58)
   //make sure we are only printing characters that we have graphics for
   //if(string[n]>32 && string[n]<90)
    g_TileMap[screen_ctr]=string[n]+offset; //draw digit
   else if(string[n]>=65 && string[n]<90)
    g_TileMap[screen_ctr]=string[n]+offset-(65-58);
   else if(string[n]>=97 && string[n]<123)
    g_TileMap[screen_ctr]=string[n]+offset-(97-58);
   else if(string[n]==46)
    g_TileMap[screen_ctr]=string[n]+offset+38;
   else if(string[n]==0)
    return;
   screen_ctr++;
 }

}

